@@ -7,6 +7,9 @@
 /* Define this if berkeley unix-domain sockets are available */
 #undef HAVE_UNIX_DOMAIN_SOCKETS
 
+/* Define this if have sun_len member in struct sockaddr_un */
+#undef HAVE_SOCKADDR_SUN_LEN
+
 /* Define this if using Xauth authentication */
 #undef HAVE_XAUTH
 
