@@ -53,6 +53,7 @@ extern void cygwin_conv_to_posix_path(const char *path, char *posix_path);
 #endif
 
 #include "gnuserv.h"
+#include "compiler.h"
 
 char gnuserv_version[] = "gnuclient version " GNUSERV_VERSION;
 
@@ -73,7 +74,7 @@ main (int argc, char *argv[])
 } /* main */
 #else /* SYSV_IPC || UNIX_DOMAIN_SOCKETS || INTERNET_DOMAIN_SOCKETS */
 
-static char cwd[PATH_MAX+2];	/* current working directory when calculated */
+static char cwd[QXE_PATH_MAX+2];/* current working directory when calculated */
 static char *cp = NULL;		/* ptr into valid bit of cwd above */
 
 static pid_t emacs_pid;			/* Process id for emacs process */
@@ -85,7 +86,7 @@ static pid_t emacs_pid;			/* Process id for emacs process */
 void initialize_signals (void);
 
 static void
-tell_emacs_to_resume (int sig)
+tell_emacs_to_resume (int UNUSED (sig))
 {
   char buffer[GSERV_BUFSZ+1];
   int s;			/* socket / msqid to server */
@@ -152,7 +153,7 @@ get_current_working_directory (void)
   if (cp == NULL)
     {				/* haven't calculated it yet */
 #ifdef HAVE_GETCWD
-      if (getcwd (cwd,PATH_MAX) == NULL)
+      if (getcwd (cwd,QXE_PATH_MAX) == NULL)
 #else
       if (getwd (cwd) == 0)
 #endif /* HAVE_GETCWD */
@@ -185,12 +186,12 @@ filename_expand (char *fullpath, char *filename)
   /* filename - filename to expand */
 {
 #ifdef  CYGWIN
-  char cygwinFilename[PATH_MAX+1];
+  char cygwinFilename[QXE_PATH_MAX+1];
   extern void cygwin_conv_to_posix_path(const char *, char *);
 #endif
 
   int len;
-  fullpath[0] = '\0';
+  fullpath[0] = fullpath[QXE_PATH_MAX] = '\0';
 
 #ifdef  CYGWIN
   /*
@@ -203,7 +204,7 @@ filename_expand (char *fullpath, char *filename)
   if (filename[0] && filename[0] == '/')
      {
        /* Absolute (unix-style) pathname.  Do nothing */
-       strcat (fullpath, filename);
+       strncat (fullpath, filename, QXE_PATH_MAX);
      }
   else
     {
@@ -211,15 +212,18 @@ filename_expand (char *fullpath, char *filename)
        and prepend it.  FIXME: need to fix the case of DOS paths like
        "\foo", where we need to get the current drive. */
 
-      strcat (fullpath, get_current_working_directory ());
+      strncat (fullpath, get_current_working_directory (), QXE_PATH_MAX);
       len = strlen (fullpath);
 
-      if (len > 0 && fullpath[len-1] == '/')	/* trailing slash already? */
-	;					/* yep */
-      else
-	strcat (fullpath, "/");		/* nope, append trailing slash */
+       /* If no trailing slash, add one */
+       if (len <= 0 || (fullpath[len - 1] != '/' && len < QXE_PATH_MAX))
+	{
+	  strcat (fullpath, "/");
+	  len++;
+	}
+
       /* Don't forget to add the filename! */
-      strcat (fullpath,filename);
+      strncat (fullpath, filename, QXE_PATH_MAX - len);
     }
 } /* filename_expand */
 
@@ -300,8 +304,8 @@ int
 main (int argc, char *argv[])
 {
   int starting_line = 1;	/* line to start editing at */
-  char command[PATH_MAX+50];	/* emacs command buffer */
-  char fullpath[PATH_MAX+1];	/* full pathname to file */
+  char command[QXE_PATH_MAX+50];/* emacs command buffer */
+  char fullpath[QXE_PATH_MAX+1];/* full pathname to file */
   char *eval_form = NULL;	/* form to evaluate with `-eval' */
   char *eval_function = NULL;	/* function to evaluate with `-f' */
   char *load_library = NULL;	/* library to load */
@@ -321,7 +325,7 @@ main (int argc, char *argv[])
   char *hostarg = NULL;		/* remote hostname */
   char *remotearg;
   char thishost[HOSTNAMSZ];	/* this hostname */
-  char remotepath[PATH_MAX+1]; /* remote pathname */
+  char remotepath[QXE_PATH_MAX+1]; /* remote pathname */
   int rflg = 0;			/* pathname given on cmdline */
   char *portarg;
   unsigned short port = 0;	/* port to server */
@@ -435,7 +439,7 @@ main (int argc, char *argv[])
 		  break;
 		case 'r':
 		  GET_ARGUMENT (remotearg, "-r");
-		  strcpy (remotepath, remotearg);
+		  strncpy (remotepath, remotearg, QXE_PATH_MAX);
 		  rflg = 1;
 		  break;
 #endif /* INTERNET_DOMAIN_SOCKETS */
@@ -590,7 +594,7 @@ main (int argc, char *argv[])
 					 * to this machine */
 	      if ((ptr = getenv ("GNU_NODE")) != NULL)
 		/* user specified a path */
-		strcpy (remotepath, ptr);
+		strncpy (remotepath, ptr, QXE_PATH_MAX);
 	    }
 #if 0  /* This is really bogus... re-enable it if you must have it! */
 #if defined (hp9000s300) || defined (hp9000s800)
