@@ -1,8 +1,8 @@
 /* -*-C-*-
  Server code for handling requests from clients and forwarding them
- on to the GNU Emacs process.
+ on to the XEmacs process.
 
- This file is part of GNU Emacs.
+ This file is part of XEmacs.
 
  Copying is permitted under those conditions described by the GNU
  General Public License.
@@ -59,9 +59,11 @@ char gnuserv_version[] = "gnuserv version" GNUSERV_VERSION;
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#include "compiler.h"
+
 #if !defined(SYSV_IPC) && !defined(UNIX_DOMAIN_SOCKETS) && \
     !defined(INTERNET_DOMAIN_SOCKETS)
-main ()
+int main ()
 {
   fprintf (stderr,"Sorry, the Emacs server is only supported on systems that have\n");
   fprintf (stderr,"Unix Domain sockets, Internet Domain sockets or System V IPC\n");
@@ -166,7 +168,7 @@ ipc_init (struct msgbuf **msgpp)
 
 /*
   handle_ipc_request -- accept a request from a client, pass the request on
-  			to the GNU Emacs process, then wait for its reply and
+  			to the XEmacs process, then wait for its reply and
 			pass that on to the client.
 */
 void
@@ -378,7 +380,7 @@ handle_response (void)
 
 #ifdef INTERNET_DOMAIN_SOCKETS
 struct entry {
-  u_long host_addr;
+  unsigned long host_addr;
   struct entry *next;
 };
 
@@ -450,7 +452,7 @@ timed_read (int fd, char *buf, int max, int timeout, int one_line)
   permitted -- return whether a given host is allowed to connect to the server.
 */
 static int
-permitted (u_long host_addr, int fd)
+permitted (unsigned long host_addr, int fd)
 {
   int key;
   struct entry *entry;
@@ -458,8 +460,6 @@ permitted (u_long host_addr, int fd)
   char auth_protocol[128];
   char buf[1024];
   int  auth_data_len;
-  int  auth_data_pos;
-  int  auth_mismatches;
 
   if (fd > 0)
     {
@@ -487,38 +487,42 @@ permitted (u_long host_addr, int fd)
 	   * doing magic cookie auth
 	   */
 
-	  if (timed_read(fd, buf, 10, AUTH_TIMEOUT, 1) <= 0)
+	  if (timed_read (fd, buf, 10, AUTH_TIMEOUT, 1) <= 0)
 	    return FALSE;
 
-	  auth_data_len = atoi(buf);
+	  auth_data_len = atoi (buf);
 
-	  if (auth_data_len <= 0 || auth_data_len > sizeof(buf))
+	  if (auth_data_len <= 0 || auth_data_len > (int) sizeof (buf))
 	      {
 		return FALSE;
 	      }
 
-	  if (timed_read(fd, buf, auth_data_len, AUTH_TIMEOUT, 0) != auth_data_len)
+	  if (timed_read (fd, buf, auth_data_len, AUTH_TIMEOUT, 0) !=
+	      auth_data_len)
 	    return FALSE;
 
 #ifdef AUTH_MAGIC_COOKIE
 	  if (server_xauth && server_xauth->data)
-	  {
+	    {
 	    /* Do a compare without comprising info about
 	       the size of the cookie */
-	    auth_mismatches =
+	    int auth_data_pos;
+	    int auth_mismatches =
 	      ( auth_data_len ^
 		server_xauth->data_length );
 
-	    for(auth_data_pos=0; auth_data_pos < auth_data_len; ++auth_data_pos)
+	    for(auth_data_pos = 0; auth_data_pos < auth_data_len;
+		++auth_data_pos)
 	      auth_mismatches |=
 		( buf[auth_data_pos] ^
-		  server_xauth->data[auth_data_pos % server_xauth->data_length]);
+		  server_xauth->data[auth_data_pos %
+				     server_xauth->data_length]);
 
 	    if (auth_mismatches == 0)
 	      return TRUE;
 	    
 	    for(;rand() % 1000;);
-	  }
+	    }
 
 #else
 	  printf ("client tried Xauth, but server is not compiled with Xauth\n");
@@ -560,7 +564,7 @@ permitted (u_long host_addr, int fd)
               already there.
 */
 static void
-add_host (u_long host_addr)
+add_host (unsigned long host_addr)
 {
   int key;
   struct entry *new_entry;
@@ -596,7 +600,7 @@ setup_table (void)
   FILE *host_file;
   char *file_name;
   char hostname[HOSTNAMSZ];
-  u_int host_addr;
+  unsigned int host_addr;
   int i, hosts=0;
 
   /* Make sure every entry is null */
@@ -605,7 +609,7 @@ setup_table (void)
 
   gethostname(hostname,HOSTNAMSZ);
 
-  if ((host_addr = internet_addr(hostname)) == -1)
+  if ((host_addr = internet_addr (hostname)) == (unsigned int) -1)
     {
       fprintf(stderr,"%s: unable to find %s in /etc/hosts or from YP",
 	      progname,hostname);
@@ -631,9 +635,10 @@ setup_table (void)
        (host_file = fopen(file_name,"r")) != NULL))	/* opened ok */
     {
       while ((fscanf(host_file,"%s",hostname) != EOF))	/* find a host */
-	if ((host_addr = internet_addr(hostname)) != -1)/* get its addr */
+	if ((host_addr = internet_addr(hostname)) != (unsigned int) -1)
+	  /* get its addr */
 	  {
-	    add_host(host_addr);				/* add the addr */
+	    add_host(host_addr);			/* add the addr */
 	    hosts++;
 	  }
       fclose(host_file);
@@ -852,7 +857,7 @@ handle_unix_request (int ls)
 
 
 int
-main (int argc, char *argv[])
+main (int UNUSED (argc), char *argv[])
 {
   int chan;			/* temporary channel number */
 #ifdef SYSV_IPC
@@ -867,11 +872,20 @@ main (int argc, char *argv[])
   for(chan=3; chan < _NFILE; close(chan++)) /* close unwanted channels */
     ;
 
+
+#ifdef WIN32_NATIVE
+  tmpdir = getenv ("TEMP");
+  if (!tmpdir)
+    tmpdir = getenv ("TMP");
+  if (!tmpdir)
+    tmpdir = "c:\\";
+#else
 #ifdef USE_TMPDIR
-  tmpdir = getenv("TMPDIR");
+  tmpdir = getenv ("TMPDIR");
 #endif
   if (!tmpdir)
     tmpdir = "/tmp";
+#endif /* WIN32_NATIVE */
 #ifdef USE_LITOUT
   {
     /* this is to allow ^D to pass to emacs */
